package hero.session;

/**
 *
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along wit this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *
--------------------------------------------------------------------------
 * $Id: AdminSessionBean.java,v 1.13 2005/10/17 15:40:34 brice Exp $
 *
--------------------------------------------------------------------------
 */
import hero.hook.NodeHookI;
import hero.hook.ProcessHookI;
import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnProjectLocalHome;
import hero.interfaces.BnNodeLocalHome;
import hero.interfaces.BnNodeLocal;
import hero.interfaces.Constants;
import hero.util.EventConstants;
import hero.util.HeroException;
import hero.util.ServerType;

import java.io.File;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;

import javax.ejb.CreateException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import org.apache.log4j.Logger;


/**
*
* The Admin Session Bean, is an stateful session bean that provides the admin API to get information about
* projects and activities in which the user is admin.<br><br>
* <strong>The following lines shows a sample code to use this API in your application:<br><br></strong>
* <br>
* First of all you have to import the Admin Session files:<br>
* <br>
* import hero.interfaces.AdminSessionLocalHome;<br>
* import hero.interfaces.AdminSessionLocal;<br>
* import hero.interfaces.AdminSessionHome;<br>
* import hero.interfaces.AdminSession;<br>
* import hero.interfaces.AdminSessionUtil;<br>
* <br>
* Now, it is time to create the Admin Session instance:<br>
* <br>
* Like this if you want to use local interfaces:<br><br>
* AdminSessionLocalHome adminh = (AdminSessionLocalHome)hero.interfaces.AdminSessionUtil.getLocalHome();<br>
* AdminSessionLocal adminsession = adminh.create();<br>
* <br>
* or like this if you use remote interfaces:<br>  
* <br>
* AdminSessionHome adminh = (AdminSessionHome)hero.interfaces.AdminSessionUtil.getHome();<br>
* AdminSession adminsession = adminh.create();<br>
* <br>
* <br>
* Now you can call all Admin Sessions methods...  
*
 *
 * @ejb:bean name="AdminSession"
 *           display-name="AdminSession Bean"
 *           type="Stateless"
 *           transaction-type="Container"
 *           jndi-name="ejb/hero/AdminSession"
 *           local-jndi-name="ejb/hero/AdminSession_L"
 *
 * @ejb:ejb-ref ejb-name="BnProject"
 *              ref-name="myhero/BnProject"
 * @ejb:transaction type="Supports"
 * @ejb.permission role-name="BONITAUSER,user,SuperAdmin" 
 * @jonas.bean
 *      ejb-name="AdminSession"
 *      jndi-name="ejb/hero/AdminSession"
 * 
 * @jboss.container-configuration name="Standard Stateless SessionBean for Bonita"
 **/

public class AdminSessionBean implements SessionBean, EventConstants  {

   // -------------------------------------------------------------------------
   // Static
   // -------------------------------------------------------------------------
   // Utility variable
	private static final Logger trace = Logger.getLogger(AdminSessionBean.class);
	
	private static final String SCRIPTS_DIR = ServerType.getServerBase() + File.separator + "bonitaScripts";
   
   // -------------------------------------------------------------------------
   // Members 
   // -------------------------------------------------------------------------
   
   private SessionContext mContext;
  
   // -------------------------------------------------------------------------
   // Methods
   // -------------------------------------------------------------------------  

   /**
	* Get admin workflow models (admin is the role of the model administrator)
	* 
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Supports"
    *
   **/        
	   public Collection getModels() throws HeroException {
	   trace.debug("start by " + mContext.getCallerPrincipal().getName());
	    Collection result = new ArrayList();
		BnProjectLocalHome projhome;
		try {
			projhome = hero.interfaces.BnProjectUtil.getLocalHome();			
			Collection models =projhome.findByAdmin(mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.MODEL);
			Iterator mdls = models.iterator(); 
			while (mdls.hasNext())
			{
				BnProjectLocal pj = (BnProjectLocal)mdls.next();
				result.add(pj.getBnProjectLightValue());
			}
			return result;
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}
	   }
	   
   /**
	* Get instances list for models in which this user is admin (admin is the role of the model administrator)
	* 
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Supports"
    *
   **/        
	   public Collection getInstances() throws HeroException {
	   trace.debug("start by " + mContext.getCallerPrincipal().getName());
		BnProjectLocalHome projhome;
		Collection result = new ArrayList();
		try {
			projhome = hero.interfaces.BnProjectUtil.getLocalHome();
			Collection instances =projhome.findByAdmin(mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
			Iterator inst = instances.iterator(); 
			while (inst.hasNext())
			{
				BnProjectLocal pj = (BnProjectLocal)inst.next();
				result.add(pj.getBnProjectLightValue());
			}
			return result;
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}
	   }
	   
   /**
	* Get activities for instances/cooperatives projects in which this user is admin (admin is the role of the model administrator)
	* 
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Supports"
	*
   **/
	   public Collection getActivities() throws HeroException {
	   trace.debug("start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocalHome nodehome;
		Collection result = new ArrayList();
		try {
			nodehome = hero.interfaces.BnNodeUtil.getLocalHome();
			Collection activities = nodehome.findAdminActivities(mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE, Constants.Pj.COOPERATIVE);
			Iterator act = activities.iterator();
			while (act.hasNext())
			{
				BnNodeLocal nd = (BnNodeLocal)act.next();
				result.add(nd.getBnNodeValue());				
			}
			return result;
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}
	  }	   	
	   
	   /**
	    * This method retrieve the list of hooks deployed on the server
	    * 
	    * @ejb:interface-method view-type="both"
	    * @ejb:transaction type="Supports"
	    *
	    **/
	   public Collection getJavaHooks(String eventType){
		   trace.debug("start by " + mContext.getCallerPrincipal().getName());
		   NodeHookI ndh;
		   ProcessHookI ph;
		   String event="";
		   try{
			   ArrayList names = new ArrayList();
			   String HOOKS_DIR = SCRIPTS_DIR+File.separator+"hero"+File.separator+"hook";
			   if ((new File(HOOKS_DIR)).exists())
			   {
				   File dir = new File(HOOKS_DIR);
				   Collection output = Arrays.asList(dir.list());
				   Iterator i = output.iterator();
				   while (i.hasNext())
				   {
					   String className = (String)i.next();
					   int index = className.indexOf(".class");
					   className=className.substring(0,index);
					   ClassLoader cl = Thread.currentThread().getContextClassLoader();					   
					   Class clhook=cl.loadClass("hero.hook."+className);
					   Class[] param={};
					   Object[] o={};
					   Method evth=clhook.getMethod("getMetadata",param);
					   if (NodeHookI.class.isAssignableFrom(clhook))
					   {
						   ndh =(NodeHookI)clhook.newInstance();
						   event=(String)evth.invoke(ndh,o);
					   }
					   else if (ProcessHookI.class.isAssignableFrom(clhook))
					   {
						   ph =(ProcessHookI)clhook.newInstance();
						   event=(String)evth.invoke(ph,o);
					   }					   
					   if (event.equals(eventType))
						   names.add(className);
				   }
				   return names;				   				   				   				
			   }			
			   else
				   return (new ArrayList());
			   
		   } catch (Exception e) {e.printStackTrace();return new ArrayList();
		   }        		   
	   }   
	   
	   /**
	    * This method retrieve the list of mappers deployed on the server
	    * 
	    * @ejb:interface-method view-type="both"
	    * @ejb:transaction type="Supports"
	    *
	    **/
	   public Collection getCustomMappers() throws HeroException {
		   trace.debug("start by " + mContext.getCallerPrincipal().getName());
		   
		   try{
			   ArrayList names = new ArrayList();
			   String MAPPER_DIR = SCRIPTS_DIR+File.separator+"hero"+File.separator+"mapper";
			   if ((new File(MAPPER_DIR)).exists())
			   {
				   File dir = new File(MAPPER_DIR);
				   Collection output = Arrays.asList(dir.list());
				   Iterator i = output.iterator();
				   while (i.hasNext())
				   {
					   String className = (String)i.next();
					   int index = className.indexOf(".class");
					   names.add(className.substring(0,index));
				   }
				   return names;				   				   				   				
			   }			
			   else
				   return (new ArrayList());
			   
		   } catch (Exception e) {
			   throw new HeroException(e.getMessage());
		   }        		   
	   }   
	  
	   /**
	    * This method retrieve the list of performer assignments deployed on the server
	    * 
	    * @ejb:interface-method view-type="both"
	    * @ejb:transaction type="Supports"
	    *
	    **/
	   public Collection getCallbackPerformers() throws HeroException {
		   trace.debug("start by " + mContext.getCallerPrincipal().getName());
		   
		   try{
			   ArrayList names = new ArrayList();
			   String PERFORMER_DIR = SCRIPTS_DIR+File.separator+"hero"+File.separator+"performerAssign";
			   if ((new File(PERFORMER_DIR)).exists())
			   {
				   File dir = new File(PERFORMER_DIR);
				   Collection output = Arrays.asList(dir.list());
				   Iterator i = output.iterator();
				   while (i.hasNext())
				   {
					   String className = (String)i.next();
					   int index = className.indexOf(".class");
					   names.add(className.substring(0,index));
				   }
				   return names;				   				   				   				
			   }			
			   else
				   return (new ArrayList());
			   
		   } catch (Exception e) {
			   throw new HeroException(e.getMessage());
		   }        		   
	   }   
	   
	   /**
	    * This method retrieve the list of Initiator Mappers deployed on the server
	    * 
	    * @ejb:interface-method view-type="both"
	    * @ejb:transaction type="Supports"
	    *
	    **/
	   public Collection getCustomInitiators() throws HeroException {
		   trace.debug("start by " + mContext.getCallerPrincipal().getName());
		   
		   try{
			   ArrayList names = new ArrayList();
			   String INITIATOR_DIR = SCRIPTS_DIR+File.separator+"hero"+File.separator+"initiatorMapper";
			   if ((new File(INITIATOR_DIR)).exists())
			   {
				   File dir = new File(INITIATOR_DIR);
				   Collection output = Arrays.asList(dir.list());
				   Iterator i = output.iterator();
				   while (i.hasNext())
				   {
					   String className = (String)i.next();
					   int index = className.indexOf(".class");
					   names.add(className.substring(0,index));
				   }
				   return names;				   				   				   				
			   }			
			   else
				   return (new ArrayList());
			   
		   } catch (Exception e) {
			   throw new HeroException(e.getMessage());
		   }        		   
	   }   
	   
   /**
   * Create the AdminSession Bean
   *
   * @throws CreateException 
   *
   * @ejb.permission unchecked="yes" 
   * @ejb:create-method view-type="both"
   **/

    public void ejbCreate() throws CreateException {
	trace.debug("ejbCreate");
    }
    
    public void setSessionContext(final javax.ejb.SessionContext context) {
	mContext = context;
    }
    
    public void ejbRemove() {
    }
    
    public void ejbActivate() {
    }
    
    public void ejbPassivate() {
    }
    
}
